local AttributeName = {
	[1] = "STR",
	[2] = "AGI",
	[3] = "STA",
	[4] = "INT",
	[5] = "MND",
	
	[6] = "MELEE_MAIN_DAMAGE",
	[7] = "MELEE_ATTACK",
	[8] = "MELEE_CRITICAL",
	[9] = "PHYSICAL_HIT",
	
	[10] = "RANGE_DAMAGE",
	[11] = "RANGE_ATTACK",
	[12] = "RANGE_CRITICAL",
	[13] = "PHYSICAL_HIT",
	
	[14] = "PHYSICAL_DEFENCE",
	[15] = "PHYSICAL_PARRY",
	[16] = "PHYSICAL_DODGE",
	[17] = "PHYSICAL_RESIST_CRITICAL",
	
	[18] = "MAGIC_DAMAGE",
	[19] = "MAGIC_ATTACK",
	[20] = "MAGIC_CRITICAL",
	[21] = "MAGIC_HEAL",
	[22] = "MAGIC_HIT",
	
	[23] = "MAGIC_DEFENCE",
	[24] = "MAGIC_DODGE",
	[25] = "MAGIC_RESIST_CRITICAL"
	};
	
local AttributeLabel = {
	[1] = C_STR,
	[2] = C_AGI,
	[3] = C_STA,
	[4] = C_INT,
	[5] = C_MND,
	
	[6] = C_PHYSICAL_DAMAGE,
	[7] = C_PHYSICAL_ATTACK,
	[8] = C_PHYSICAL_CRITICAL,
	[9] = C_PHYSICAL_HIT,
	
	[10] = C_PHYSICAL_DAMAGE,
	[11] = C_PHYSICAL_ATTACK,
	[12] = C_PHYSICAL_CRITICAL,
	[13] = C_PHYSICAL_HIT,
	
	[14] = C_PHYSICAL_DEFENCE,
	[15] = C_PHYSICAL_PARRY,
	[16] = C_PHYSICAL_DODGE,
	[17] = C_PHYSICAL_RESIST_CRITICAL,
	
	[18] = C_MAGIC_DAMAGE,
	[19] = C_MAGIC_ATTACK,
	[20] = C_MAGIC_CRITICAL,
	[21] = C_MAGIC_HEAL_POINT,
	[22] = C_MAGIC_HIT,
	
	[23] = C_MAGIC_DEFENCE,
	[24] = C_MAGIC_DODGE,
	[25] = C_MAGIC_RESIST_CRITICAL,
	
	[26] = C_EARTH,
	[27] = C_WATER,
	[28] = C_FIRE,
	[29] = C_WIND,
	[30] = C_LIGHT,
	[31] = C_DARK,
	
	[32] = C_GOODEVIL,
	[33] = C_HONOR,
	};
	
local AttributeHead = {
	[1] = MELEE,
	[2] = RANGE,
	[3] = PHYSICAL_DEFENCE,
	[4] = MAGIC,
	[5] = MAGIC_DEFENCE
	};

function CharacterAttribute_OnLoad(this)
	this:RegisterEvent("PLAYER_ABILITY_CHANGED");
	this:RegisterEvent("PLAYER_ABILITYPOINT_CHANGED");
	this:RegisterEvent("PLAYER_GOODEVIL_CHANGED");
	this:RegisterEvent("PLAYER_HONORPOINT_CHANGED");
	this:RegisterEvent("PLAYER_EXP_CHANGED");
	
	for i = 1, 33 do
		local stringLabel = getglobal("CharacterAttributeString"..i.."Label")
		stringLabel:SetText(AttributeLabel[i]);
	end
	
	for i = 1, 5 do
		local stringLabelName = getglobal("CharacterAttributeLabel"..i.."Name")
		stringLabelName:SetText(AttributeHead[i]);
	end
	
	CharacterAttribute_Update();
	Resistance_Update();
	GoodEvilPoint_Update();	
	HonorPoint_Update();
	ClassList_Update()
end	

function CharacterAttribute_OnEvent(this, event, arg1, arg2)
	if (event == "PLAYER_ABILITY_CHANGED" or event == "PLAYER_ABILITYPOINT_CHANGED") then
		CharacterAttribute_Update();
		Resistance_Update();
		return;
	end
	if (event == "PLAYER_GOODEVIL_CHANGED") then
		GoodEvilPoint_Update();	
	end
	if (event == "PLAYER_HONORPOINT_CHANGED") then
		HonorPoint_Update();
	end
	if (event == "PLAYER_EXP_CHANGED") then
		ClassList_Update();
	end
end

function CharacterAttribute_Update()
	for i = 1, 25 do
		local stringValue = getglobal("CharacterAttributeString"..i.."Value")
		local baseValue, otherValue = GetPlayerAbility(AttributeName[i]);
		local value = baseValue + otherValue
		if (otherValue > 0) then
			stringValue:SetText(GREEN_FONT_COLOR_CODE..value..FONT_COLOR_CODE_CLOSE.." ("..baseValue..GREEN_FONT_COLOR_CODE.."+"..otherValue..FONT_COLOR_CODE_CLOSE..")");
		elseif (otherValue < 0) then
			stringValue:SetText(RED_FONT_COLOR_CODE..value..FONT_COLOR_CODE_CLOSE.." ("..baseValue..RED_FONT_COLOR_CODE.."-"..otherValue..FONT_COLOR_CODE_CLOSE..")");
		elseif (value == 0) then
			stringValue:SetText("0 (0)");
		else
			stringValue:SetText(value.." ("..baseValue.."+"..otherValue..")");
		end
	end
end

function Resistance_Update()
	local earth, water, fire, wind, light, dark = GetPlayerResistance();
	CharacterAttributeString26Value:SetText(earth);
	CharacterAttributeString27Value:SetText(water);
	CharacterAttributeString28Value:SetText(fire);
	CharacterAttributeString29Value:SetText(wind);
	CharacterAttributeString30Value:SetText(light);
	CharacterAttributeString31Value:SetText(dark);
end

function GoodEvilPoint_Update()
	local value, type = GetPlayerGoodEvil();
	if (value > -0.01 and value < 0) then
		value = -0.01;
	end
	CharacterAttributeString32Value:SetText(string.format("%.2f", value));
	CharacterAttributeString32Value:SetColor(GetGoodEvilTypeColor(type));
end

function HonorPoint_Update()
	CharacterAttributeString33Value:SetText(GetPlayerMedalCount());
end

function ClassList_Update()
	local count, maxNums = GetPlayerNumClasses();
	for i = 1, 9 do
		local className = getglobal("ClassListItem"..i.."Name");
		local classExp = getglobal("ClassListItem"..i.."Exp");
		if (i <= count) then
			class, token, level, currValue, maxValue = GetPlayerClassInfo(i, true);
			className:SetText(class);
			classExp:SetText(currValue.." / "..maxValue);
		else
			className:SetText("");
			classExp:SetText("");
		end
	end
end