SUI_Settings = {};
SUI_MOB_HEALTH = dofile("interface/addons/fSlimUI/UnitFrames/SUI_MobsHealth.lua");

HealthBarColor = {r = 1.00, g = 0.00, b = 0.00};
SkillBarColor = {
	[1] = {r = 0.00, g = 0.00, b = 1.00, prefix = "MANA"};
	[2] = {r = 1.00, g = 1.00, b = 0.00, prefix = "FURY_POINTS"};
	[3] = {r = 0.00, g = 1.00, b = 0.00, prefix = "CONCENTRATION_POINTS"};
	[4] = {r = 0.50, g = 0.00, b = 1.00, prefix = "VIGOR_POINTS"}
};

SUI_ClassColor = {   
	["WARRIOR"] = { r = 0.984 , g = 0.255 , b = 0.024 },    
	["RANGER"]	= { r = 0.647 , g = 0.839 , b = 0.011 },
	["THIEF"]	= { r = 0.000 , g = 0.839 , b = 0.773 },  
	["MAGE"]	= { r = 0.988 , g = 0.447 , b = 0.067 },   
	["AUGUR"]	= { r = 0.157 , g = 0.549 , b = 0.925 },       	         
	["KNIGHT"]	= { r = 0.878 , g = 0.886 , b = 0.294 },        
	["WARDEN"]	= { r = 0.757 , g = 0.290 , b = 0.819 },             
	["DRUID"]	= { r = 0.380 , g = 0.819 , b = 0.378 }
};

SUI_RaceColor = {   
	["Boss"]	= { r = 0.5 , g = 0 , b = 1 },    
	["Elite"]	= { r = 1 , g = 0.5 , b = 0 },
	["Race"]	= { r = 1 , g = 1 , b = 0 }
};

function SUI_UnitFrame_Initialize(this, unit, name, portrait, healthBar, manaBar, skillBar, raidtarget)
	this.unit = unit;
	this.name = name;
	this.portrait = portrait;
	this.healthBar = healthBar;
	this.manaBar = manaBar;
	this.skillBar = skillBar;
	this.raidtarget	= raidtarget;

	SUI_UnitFrameHealthBar_Initialize(this.healthBar, unit);
	SUI_UnitFrameManaBar_Initialize(this.manaBar, unit);
	SUI_UnitFrameSkillBar_Initialize(this.skillBar, unit);
	
	this:RegisterEvent("UNIT_NAME_UPDATE");
	this:RegisterEvent("UNIT_LEVEL");
	this:RegisterEvent("UNIT_PORTRAIT_UPDATE");
	this:RegisterEvent("RAID_TARGET_CHANGED");
	
	SUI_UnitFrame_Update(this);
end

function SUI_UnitFrame_OnEvent(this, event, arg1, arg2)
	if (event == "UNIT_NAME_UPDATE") then
		if (arg1 == this.unit and this.name) then
			this.name:SetText(UnitName(this.unit));
		end
	elseif (event == "UNIT_PORTRAIT_UPDATE") then
		if (arg1 == this.unit) then
			SUI_UnitFramePortrait_Update(this.portrait, this.unit);
		end
	elseif (event == "UNIT_LEVEL") then
		if (arg1 == this.unit) then
			SUI_UnitFrameRace_Update(this.healthBar, this.unit);
			SUI_UnitFrameClass_Update(this.manaBar, this.unit);
			SUI_UnitFrameSubClass_Update(this.skillBar, this.unit);
		end
	elseif (event == "RAID_TARGET_CHANGED") then
		if (this:IsVisible()) then
			RaidTargetFrame_Update(this.raidtarget, this.unit);
		end
	end
end

function SUI_UnitFramePortrait_Update(portrait, unit)
	if (portrait) then
		portrait:SetUnit(unit, nil, true);
	end
end

function SUI_UnitFrame_Update(this)
	if (this.name) then
		this.name:SetText(UnitName(this.unit));
		if string.find(this.unit, "party") then
			local color = SUI_ClassColor[UnitClassToken(this.unit)]
			if color then
				this.name:SetColor(color.r, color.g, color.b)
			end
		end
	end
	
	RaidTargetFrame_Update(this.raidtarget, this.unit);
	SUI_UnitFramePortrait_Update(this.portrait, this.unit);
	SUI_UnitFrameHealthBar_Update(this.healthBar, this.unit);
	SUI_UnitFrameManaBar_Update(this.manaBar, this.unit);
	SUI_UnitFrameSkillBar_Update(this.skillBar, this.unit);
	SUI_UnitFrameRace_Update(this.healthBar, this.unit);
	SUI_UnitFrameClass_Update(this.manaBar, this.unit);
	SUI_UnitFrameSubClass_Update(this.skillBar, this.unit);

	if (this.target) then
		local target = this.target;
		if (UnitExists(this.unit) and UnitExists(target.unit)) then
			if (target.AutoShow) then
				target:Show();
			end
			SUI_UnitFrame_Update(target);
		else
			SUI_UnitFrame_Update(target);
			if (target.AutoShow) then
				target:Hide();
			end
		end
	end
end

function SUI_UnitFrameHealthBar_Initialize(statusBar, unit)
	statusBar.unit = unit;
	statusBar:RegisterEvent("UNIT_HEALTH");
	statusBar:RegisterEvent("UNIT_MAXHEALTH");
end

function SUI_UnitFrameManaBar_Initialize(statusBar, unit)
	statusBar.unit = unit;
	statusBar:RegisterEvent("UNIT_MANA");
	statusBar:RegisterEvent("UNIT_MAXMANA");
end

function SUI_UnitFrameSkillBar_Initialize(statusBar, unit)
	statusBar.unit = unit;
	statusBar:RegisterEvent("UNIT_SKILL");
	statusBar:RegisterEvent("UNIT_MAXSKILL");
end

function SUI_MobHealth_Update(unit)  -- like pbInfo, thanks to novayuna
	local name = UnitName(unit);
	local level = UnitLevel(unit);
	local currentHP = UnitChangeHealth(unit);
	if (type(SUI_MOB_HEALTH[name]) ~= "table") then
		SUI_MOB_HEALTH[name] = {};
	end
	if (type(SUI_MOB_HEALTH[name][level]) ~= "table") then
		SUI_MOB_HEALTH[name][level] = {};
	end
	if (UnitHealth(unit) == 100 and (SUI_MOB_HEALTH[name][level]["curHP"] or 0) < currentHP) then
		SUI_MOB_HEALTH[name][level]["curHP"] = currentHP;
	end
end
					
function SUI_UnitFrameHealthBar_Update(statusBar, unit)
	if (unit == statusBar.unit) then
		local info = HealthBarColor;
		statusBar:SetBarColor(info.r, info.g, info.b);
		statusBar:SetMaxValue(UnitMaxHealth(unit));
		statusBar:SetValue(UnitHealth(unit));
		local text = getglobal(statusBar:GetName().."ValueText");
		if (text) then
			if UnitExists(unit) and not UnitIsPlayer(unit) and not UnitMaster(unit) and UnitLevel(unit) > 0 then
				SUI_MobHealth_Update(unit);
				local healthMax = SUI_MOB_HEALTH[UnitName(unit)][UnitLevel(unit)]["curHP"] or 0;
				local healthRatio = UnitHealth(unit);
				if (type(healthMax) == "number" and healthMax > 0) then
					text:SetText(SUI_Value(math.ceil((healthRatio * healthMax) / 100)).." / "..SUI_Value(healthMax).." ("..healthRatio.."%)");
				else
					text:SetText(healthRatio.."%");
				end
				text:Show();
			else
				local maxHealth = UnitMaxHealth(unit);
				local curHealth = UnitHealth(unit);
				if UnitIsPlayer(unit) or UnitMaster(unit) then
					text:SetText(SUI_Value(curHealth).." / "..SUI_Value(maxHealth).." ("..math.ceil((curHealth * 100 ) / maxHealth).."%)");
				else
					text:SetText(curHealth.."%");
				end
				text:Show();
			end
		end
	end
end
		
function SUI_UnitFrameManaBar_Update(statusBar, unit)
	if (unit == statusBar.unit) then
		local maxValue = UnitMaxMana(unit);
		local currValue = UnitMana(unit);
		local info = SkillBarColor[UnitManaType(unit)];
		statusBar:SetBarColor(info.r, info.g, info.b);
		statusBar:SetMaxValue(maxValue);
		statusBar:SetValue(currValue);
		local text = getglobal(statusBar:GetName().."ValueText");
		if (text) then
			if (UnitIsPlayer(unit) or UnitMaster(unit)) then
				if maxValue <= 0 then
					text:SetText("");
				else
					text:SetText(SUI_Value(currValue).." / "..SUI_Value(maxValue).." ("..math.ceil((currValue * 100 ) / maxValue).."%)");
				end
				text:Show();
			else
				if maxValue <= 0 then
					text:SetText("");
				else
					text:SetText(currValue.."%");
				end
				text:Show();
			end
		end
	end
end

function SUI_UnitFrameSkillBar_Update(statusBar, unit)
	if (unit == statusBar.unit) then
		local maxValue = UnitMaxSkill(unit);
		local currValue = UnitSkill(unit);
		local info = SkillBarColor[UnitSkillType(unit)];
		statusBar:SetBarColor(info.r, info.g, info.b);
		statusBar:SetMaxValue(maxValue);
		statusBar:SetValue(currValue);
		local text = getglobal(statusBar:GetName().."ValueText");
		if (text) then
			if (UnitIsPlayer(unit) or UnitMaster(unit)) then
				if maxValue <= 0 then
					text:SetText("");
				else
					text:SetText(SUI_Value(currValue).." / "..SUI_Value(maxValue).." ("..math.ceil((currValue * 100 ) / maxValue).."%)");
				end
				text:Show();
			else
				if maxValue <= 0 then
					text:SetText("");
				else
					text:SetText(currValue.."%");
				end
				text:Show();
			end
		end
	end
end

function SUI_UnitFrameRace_Update(statusBar, unit)
	if (unit == statusBar.unit) then
		local racetext = getglobal(statusBar:GetName().."InfoText");
		if UnitSex(unit) > 2 and not UnitIsPlayer(unit) then
			racetext:SetColor(SUI_RaceColor["Boss"].r, SUI_RaceColor["Boss"].g, SUI_RaceColor["Boss"].b);
			racetext:SetText(SUI_BOSS);
		elseif (UnitSex(unit) == 2) and not UnitIsPlayer(unit) then
			racetext:SetColor(SUI_RaceColor["Elite"].r, SUI_RaceColor["Elite"].g, SUI_RaceColor["Elite"].b);
			racetext:SetText(C_ELITE);
		else
			racetext:SetColor(SUI_RaceColor["Race"].r, SUI_RaceColor["Race"].g, SUI_RaceColor["Race"].b);
			racetext:SetText(UnitRace(unit));
		end
	end
end

function SUI_UnitFrameClass_Update(statusBar, unit)
	if (unit == statusBar.unit) then
		local classtext = getglobal(statusBar:GetName().."InfoText");
		local mainClass, secClass = UnitClass(unit);
		local mainLevel, secLevel = UnitLevel(unit);
		local mainToken, subToken = UnitClassToken(unit);
		local color = SUI_ClassColor[mainToken]
		if (mainLevel) then
			if (mainLevel < 1) then
				classtext:SetText("");
			else
				classtext:SetColor(color.r, color.g, color.b);
				classtext:SetText(string.sub(mainClass, 1, 2).."-"..mainLevel);
			end
		end
	end
end

function SUI_UnitFrameSubClass_Update(statusBar, unit)
	if (unit == statusBar.unit) then
		local classtext = getglobal(statusBar:GetName().."InfoText");
		local mainClass, secClass = UnitClass(unit);
		local mainLevel, secLevel = UnitLevel(unit);
		local mainToken, subToken = UnitClassToken(unit);
		local color = SUI_ClassColor[subToken]
		if (secLevel) then
			if (secLevel < 1) then
				classtext:SetText("");
			elseif color then
				classtext:SetColor(color.r, color.g, color.b);
				classtext:SetText(string.sub(secClass, 1, 2).."-"..secLevel);
			end
		end
	end
end

function SUI_MoveableFrame_OnLoad(this)
	this:RegisterEvent("VARIABLES_LOADED");
	SaveVariables("SUI_Settings");
end

function SUI_MoveableFrame_OnEvent(this, event)
	if (event == "VARIABLES_LOADED") then
		local frameName = tostring(this:GetName());
		if (type(SUI_Settings[frameName]) ~= "table") then
			SUI_Settings[frameName] = {};
		end
		if not SUI_Settings[frameName]["X"] or not SUI_Settings[frameName]["Y"] then
			SUI_Settings[frameName]["X"], SUI_Settings[frameName]["Y"] = this:GetPos();
			this:ClearAllAnchors();
			this:SetAnchor("TOPLEFT", "TOPLEFT", "UIParent", SUI_Settings[frameName]["X"], SUI_Settings[frameName]["Y"]);
		else
			local scale = GetUIScale();
			this:ClearAllAnchors();
			this:SetAnchor("TOPLEFT", "TOPLEFT", "UIParent", SUI_Settings[frameName]["X"]/scale, SUI_Settings[frameName]["Y"]/scale);   
		end
	end
end

function SUI_MoveableFrame_OnMouseDown(this)
	if IsShiftKeyDown() then
		this:StartMoving();
	end
end

function SUI_MoveableFrame_OnMouseUp(this)
	local frameName = tostring(this:GetName());
	this:StopMovingOrSizing();
	SUI_Settings[frameName]["X"], SUI_Settings[frameName]["Y"] = this:GetPos();
end

function HideUnitFrames_OnLoad(this)
	this:RegisterEvent("LOADING_END");
end
	
function HideUnitFrames_OnEvent(event)
    if event == "LOADING_END" then
        PlayerFrame:UnregisterEvent("PARTY_LEADER_CHANGED");
        PlayerFrame:UnregisterEvent("UPDATE_PARALLEL");
        PlayerFrame:UnregisterEvent("UPDATE_LOOT_ASSIGN");
        PlayerFrame:UnregisterEvent("UNIT_NAME_UPDATE");
    	PlayerFrame:UnregisterEvent("UNIT_LEVEL");
    	PlayerFrame:UnregisterEvent("UNIT_PORTRAIT_UPDATE");
    	PlayerFrame:UnregisterEvent("RAID_TARGET_CHANGED");
    	PlayerFrame:UnregisterEvent("UNIT_RELATION");
        PlayerFrame.healthBar:UnregisterEvent("UNIT_HEALTH");
        PlayerFrame.healthBar:UnregisterEvent("UNIT_MAXHEALTH");
        PlayerFrame.manaBar:UnregisterEvent("UNIT_MANA");
        PlayerFrame.manaBar:UnregisterEvent("UNIT_MAXMANA");
        PlayerFrame.skillBar:UnregisterEvent("UNIT_SKILL");
        PlayerFrame.skillBar:UnregisterEvent("UNIT_MAXSKILL");
        PlayerFrame:Hide();
        
        TargetFrame:UnregisterEvent("UNIT_TARGET_CHANGED");
    	TargetFrame:UnregisterEvent("UNIT_BUFF_CHANGED");
    	TargetFrame:UnregisterEvent("UNIT_CLASS_CHANGED");
        TargetFrame:UnregisterEvent("UNIT_NAME_UPDATE");
    	TargetFrame:UnregisterEvent("UNIT_LEVEL");
    	TargetFrame:UnregisterEvent("UNIT_PORTRAIT_UPDATE");
    	TargetFrame:UnregisterEvent("RAID_TARGET_CHANGED");
    	TargetFrame:UnregisterEvent("UNIT_RELATION");
        TargetFrame.healthBar:UnregisterEvent("UNIT_HEALTH");
        TargetFrame.healthBar:UnregisterEvent("UNIT_MAXHEALTH");
        TargetFrame.manaBar:UnregisterEvent("UNIT_MANA");
        TargetFrame.manaBar:UnregisterEvent("UNIT_MAXMANA");
        TargetFrame.skillBar:UnregisterEvent("UNIT_SKILL");
        TargetFrame.skillBar:UnregisterEvent("UNIT_MAXSKILL");
        TargetFrame:Hide();
        
        TargetTargetFrame:UnregisterEvent("UNIT_CLASS_CHANGED");
    	TargetTargetFrame:UnregisterEvent("UNIT_BUFF_CHANGED");
    	TargetTargetFrame:UnregisterEvent("UNIT_TARGET_CHANGED");
        TargetTargetFrame.healthBar:UnregisterEvent("UNIT_HEALTH");
        TargetTargetFrame.healthBar:UnregisterEvent("UNIT_MAXHEALTH");
        TargetTargetFrame.manaBar:UnregisterEvent("UNIT_MANA");
        TargetTargetFrame.manaBar:UnregisterEvent("UNIT_MAXMANA");
        TargetTargetFrame.skillBar:UnregisterEvent("UNIT_SKILL");
        TargetTargetFrame.skillBar:UnregisterEvent("UNIT_MAXSKILL");
        TargetTargetFrame:Hide();
        
        UnitFrame_party1:UnregisterEvent("PARTY_MEMBER_CHANGED");
    	UnitFrame_party1:UnregisterEvent("PARTY_LEADER_CHANGED");
    	UnitFrame_party1:UnregisterEvent("UNIT_CLASS_CHANGED");
    	UnitFrame_party1:UnregisterEvent("UNIT_BUFF_CHANGED");
    	UnitFrame_party1:UnregisterEvent("RAID_TARGET_CHANGED");
    	UnitFrame_party1:UnregisterEvent("LOOT_METHOD_CHANGED");
        UnitFrame_party1:UnregisterEvent("UNIT_NAME_UPDATE");
    	UnitFrame_party1:UnregisterEvent("UNIT_LEVEL");
    	UnitFrame_party1:UnregisterEvent("UNIT_PORTRAIT_UPDATE");
    	UnitFrame_party1:UnregisterEvent("UNIT_RELATION");
        UnitFrame_party1.healthBar:UnregisterEvent("UNIT_HEALTH");
        UnitFrame_party1.healthBar:UnregisterEvent("UNIT_MAXHEALTH");
        UnitFrame_party1.manaBar:UnregisterEvent("UNIT_MANA");
        UnitFrame_party1.manaBar:UnregisterEvent("UNIT_MAXMANA");
        UnitFrame_party1.skillBar:UnregisterEvent("UNIT_SKILL");
        UnitFrame_party1.skillBar:UnregisterEvent("UNIT_MAXSKILL");
        UnitFrame_party1:Hide();
        
        UnitFrame_party2:UnregisterEvent("PARTY_MEMBER_CHANGED");
    	UnitFrame_party2:UnregisterEvent("PARTY_LEADER_CHANGED");
    	UnitFrame_party2:UnregisterEvent("UNIT_CLASS_CHANGED");
    	UnitFrame_party2:UnregisterEvent("UNIT_BUFF_CHANGED");
    	UnitFrame_party2:UnregisterEvent("RAID_TARGET_CHANGED");
    	UnitFrame_party2:UnregisterEvent("LOOT_METHOD_CHANGED");
        UnitFrame_party2:UnregisterEvent("UNIT_NAME_UPDATE");
    	UnitFrame_party2:UnregisterEvent("UNIT_LEVEL");
    	UnitFrame_party2:UnregisterEvent("UNIT_PORTRAIT_UPDATE");
    	UnitFrame_party2:UnregisterEvent("UNIT_RELATION");
        UnitFrame_party2.healthBar:UnregisterEvent("UNIT_HEALTH");
        UnitFrame_party2.healthBar:UnregisterEvent("UNIT_MAXHEALTH");
        UnitFrame_party2.manaBar:UnregisterEvent("UNIT_MANA");
        UnitFrame_party2.manaBar:UnregisterEvent("UNIT_MAXMANA");
        UnitFrame_party2.skillBar:UnregisterEvent("UNIT_SKILL");
        UnitFrame_party2.skillBar:UnregisterEvent("UNIT_MAXSKILL");
        UnitFrame_party2:Hide();
        
        UnitFrame_party3:UnregisterEvent("PARTY_MEMBER_CHANGED");
    	UnitFrame_party3:UnregisterEvent("PARTY_LEADER_CHANGED");
    	UnitFrame_party3:UnregisterEvent("UNIT_CLASS_CHANGED");
    	UnitFrame_party3:UnregisterEvent("UNIT_BUFF_CHANGED");
    	UnitFrame_party3:UnregisterEvent("RAID_TARGET_CHANGED");
    	UnitFrame_party3:UnregisterEvent("LOOT_METHOD_CHANGED");
        UnitFrame_party3:UnregisterEvent("UNIT_NAME_UPDATE");
    	UnitFrame_party3:UnregisterEvent("UNIT_LEVEL");
    	UnitFrame_party3:UnregisterEvent("UNIT_PORTRAIT_UPDATE");
    	UnitFrame_party3:UnregisterEvent("UNIT_RELATION");
        UnitFrame_party3.healthBar:UnregisterEvent("UNIT_HEALTH");
        UnitFrame_party3.healthBar:UnregisterEvent("UNIT_MAXHEALTH");
        UnitFrame_party3.manaBar:UnregisterEvent("UNIT_MANA");
        UnitFrame_party3.manaBar:UnregisterEvent("UNIT_MAXMANA");
        UnitFrame_party3.skillBar:UnregisterEvent("UNIT_SKILL");
        UnitFrame_party3.skillBar:UnregisterEvent("UNIT_MAXSKILL");
        UnitFrame_party3:Hide();
        
        UnitFrame_party4:UnregisterEvent("PARTY_MEMBER_CHANGED");
    	UnitFrame_party4:UnregisterEvent("PARTY_LEADER_CHANGED");
    	UnitFrame_party4:UnregisterEvent("UNIT_CLASS_CHANGED");
    	UnitFrame_party4:UnregisterEvent("UNIT_BUFF_CHANGED");
    	UnitFrame_party4:UnregisterEvent("RAID_TARGET_CHANGED");
    	UnitFrame_party4:UnregisterEvent("LOOT_METHOD_CHANGED");
        UnitFrame_party4:UnregisterEvent("UNIT_NAME_UPDATE");
    	UnitFrame_party4:UnregisterEvent("UNIT_LEVEL");
    	UnitFrame_party4:UnregisterEvent("UNIT_PORTRAIT_UPDATE");
    	UnitFrame_party4:UnregisterEvent("UNIT_RELATION");
        UnitFrame_party4.healthBar:UnregisterEvent("UNIT_HEALTH");
        UnitFrame_party4.healthBar:UnregisterEvent("UNIT_MAXHEALTH");
        UnitFrame_party4.manaBar:UnregisterEvent("UNIT_MANA");
        UnitFrame_party4.manaBar:UnregisterEvent("UNIT_MAXMANA");
        UnitFrame_party4.skillBar:UnregisterEvent("UNIT_SKILL");
        UnitFrame_party4.skillBar:UnregisterEvent("UNIT_MAXSKILL");
        UnitFrame_party4:Hide();
        
        UnitFrame_party5:UnregisterEvent("PARTY_MEMBER_CHANGED");
    	UnitFrame_party5:UnregisterEvent("PARTY_LEADER_CHANGED");
    	UnitFrame_party5:UnregisterEvent("UNIT_CLASS_CHANGED");
    	UnitFrame_party5:UnregisterEvent("UNIT_BUFF_CHANGED");
    	UnitFrame_party5:UnregisterEvent("RAID_TARGET_CHANGED");
    	UnitFrame_party5:UnregisterEvent("LOOT_METHOD_CHANGED");
        UnitFrame_party5:UnregisterEvent("UNIT_NAME_UPDATE");
    	UnitFrame_party5:UnregisterEvent("UNIT_LEVEL");
    	UnitFrame_party5:UnregisterEvent("UNIT_PORTRAIT_UPDATE");
    	UnitFrame_party5:UnregisterEvent("UNIT_RELATION");
        UnitFrame_party5.healthBar:UnregisterEvent("UNIT_HEALTH");
        UnitFrame_party5.healthBar:UnregisterEvent("UNIT_MAXHEALTH");
        UnitFrame_party5.manaBar:UnregisterEvent("UNIT_MANA");
        UnitFrame_party5.manaBar:UnregisterEvent("UNIT_MAXMANA");
        UnitFrame_party5.skillBar:UnregisterEvent("UNIT_SKILL");
        UnitFrame_party5.skillBar:UnregisterEvent("UNIT_MAXSKILL");
        UnitFrame_party5:Hide();
    end
end

function SUI_Value(number)
	local value = math.abs(tonumber(number));
	local text = "";
	local quotient, remainder;
	if value == 0 then
		return "0";
	end
	while (value > 0) do
		quotient = math.floor(value / 1000);
		remainder = math.mod(value, 1000);	
		if (quotient > 0) then
			remainder = string.format("%03d", remainder);
		end
		if (text == "") then
			text = remainder;
		else
			text = remainder.."."..text;
		end
		value = quotient;
	end
	return text;
end